#include "lexer-v2-1.h"
#include <stdio.h>
#include "emit.h"

static FILE *output;

void save_to(char *file)
{
  output = fopen(file,"w");
  if(output == NULL)
  {
    fail("Cannot open file '%s' for output!",file);
  }
  return;
}

void fin()
{
  fclose(output);
}

void emit(int operation, ...)
{
  va_list args;
  va_start(args,operation);

  switch(operation)
    {
    case READ:

      {
      // argument is a location in the symbol table, resolved statically
      // and passed in as a parameter; we do i/o and update that location
      // this does not affect the expression stack

	int ptr = va_arg(args,int);
	fprintf(output,"scanf(\"%%d\",&util0);\n");
	fprintf(output,"location[%d] = util0;\n",ptr);

      }
      break;
    case WRITE:
      // no argument, uses bottom of expression stack, zeroes out the expression
      // stack
      fprintf(output,"printf(\"%%d\\n\",expression_stack[0]);\n");
      fprintf(output,"stackptr = 0;\n");

      break;
    case ASSIGN:
      {
      // argument is the destination location, statically resolved 
      // by the parser using the symbol table; 
      // we copy the bottom of the expression stack and zero the stack
	int ptr = va_arg(args,int);
	fprintf(output,"location[%d] = expression_stack[0];\n",ptr);
	fprintf(output,"stackptr = 0;\n");
      }
      break;

    case PUSH:
      {
	// argument is a number literal to push onto the expression stack
	int ptr = va_arg(args,int);
	fprintf(output,"expression_stack[stackptr] = %d;\n",ptr);
	fprintf(output,"stackptr++;\n");
      }
      break;

    case PUSH_FROM_MEM:
      {
	// argument is a location in memory to push onto the expression stack
	// statically resolved by the parser, we just get the memory location
	int ptr = va_arg(args,int);
	fprintf(output,"expression_stack[stackptr] = location[%d];\n",ptr);
	fprintf(output,"stackptr++;\n");
      }
      break;

    case CALL:
      {
	// argument is the name of a function to call
	char *func = va_arg(args,char *);
	fprintf(output,"%s();\n",func);
      }
      break;

    default:
      fail("emit.c emit(): Unknown operation");
    }

  va_end(args);
}
