#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "lexer-v2-1.h"
#include "emit.h"
#include "symbol_table.h"

/*
   forwards
 */
void program();
void stmt_list(char *parent);
void stmt(char *parent);
void expression(char *parent);
void expression_list(char *parent);


// utility
char *make_label();


// Let's go!

void parse_2_15_func_change(char *filename)
{

  // graphviz output
  printf("digraph { node [shape=\"record\"]\n\n");
  //

  lexer_reset(filename);
  program();

  // graphviz output
  printf("}\n\n");
  //
}


void program(char *parent)
{
  // graphviz output
  char *label = make_label();
  printf("\"node%s\" [ label = \"program | %s\" ]; \n",
	 label,
	 current_token);
  //

  if(try_match("id") ||
     try_match("read") ||
     try_match("write"))
    {
      stmt_list(label);
    }

  fprintf(stderr,"Program recognized\n");

  // graphviz
  free(label);
  //
}

void stmt_list(char *parent)
{
  // graphviz output
  char *label = make_label();
  printf("\"node%s\" [ label = \"stmt_list | %s\" ]; \n",
	 label,
	 current_token);
  printf("\"node%s\" -> \"node%s\" ;\n",parent,label);
  //

  stmt(label);
  if(current_token)
    stmt_list(label);

  
  // graphviz
  free(label);
  //
}

void stmt(char *parent)
{
  // graphviz output
  char *label = make_label();
  printf("\"node%s\" [ label = \"stmt | %s\" ]; \n",
	 label,
	 current_token);
  printf("\"node%s\" -> \"node%s\" ;\n",parent,label);
  //

  if(try_match("read"))
    {
      advance();
      if(try_match(identifier))
	{
	  int ptr = symbol_table_add(current_token);
	  emit(READ,ptr);
	  advance();
	}
    }
  else if(try_match("write"))
    {
      advance();
      expression(label);
      emit(WRITE);
    }
  else if(try_match(identifier))
    {
      int ptr = symbol_table_add(current_token);
      advance();
      must_match(":=");
      expression(label);
      emit(ASSIGN,ptr);
    }
  else
    {
      fail("Not a statement!?");
    }
  
  // semantics

  // graphviz
  free(label);
  //
}

void expression(char *parent)
{
  // graphviz output
  char *label = make_label();
  printf("\"node%s\" [ label = \"expression | %s\" ]; \n",
	 label,
	 current_token);
  printf("\"node%s\" -> \"node%s\" ;\n",parent,label);
  //

  if(try_match(identifier))
    {
      // semantics: if this is followed by (, it's a function call
      //            if not, then it's a reference to an existing identifier
      char *tmp = strdup(current_token);

      advance();

      if(try_match("[(]"))
	{
	  advance();
	  expression_list(parent);
	  must_match("[)]");
	  emit(CALL,tmp);
	}
      else
	{
	  int ptr = symbol_table_lookup(tmp);
	  if(ptr == -1)
	    fail("Unrecognized variable '%s'",tmp);
	  emit(PUSH_FROM_MEM,ptr);
	}
      free(tmp);
    }
  else if(try_match(number))
    {
      // semantics
      emit(PUSH,atoi(current_token));
      // 

      advance();
    }
  else
    {
      fail("Not an expression");
    }
      
  // graphviz
  free(label);
  //
}

void expression_list(char *parent)
{
  // graphviz output
  char *label = make_label();
  printf("\"node%s\" [ label = \"expression_list | %s\" ]; \n",
	 label,
	 current_token);
  printf("\"node%s\" -> \"node%s\" ;\n",parent,label);
  //

  if(try_match("[)]"))
    {
      return;
    }

  expression(label);
  if(try_match(","))
    {
      advance();
      expression_list(label);
    }
  
  // graphviz
  free(label);
  //
}



// utility

char *make_label()
{
  static int count = 0;
  char buffer[128];

  sprintf(buffer,"%05d",count);
  count++;

  return(strdup(buffer));
}
